@@ -1,7 +1,7 @@
 /*
     ccid.c: CCID common code
     Copyright (C) 2003-2010   Ludovic Rousseau
-    Copyright (C) 2009-2015   Advanced Card Systems Ltd.
+    Copyright (C) 2009-2016   Advanced Card Systems Ltd.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -18,10 +18,6 @@
 	Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-/*
- * $Id$
- */
-
 #include <config.h>
 
 #ifdef HAVE_STDIO_H
@@ -52,7 +48,7 @@
 #include <CoreFoundation/CoreFoundation.h>
 #endif
 
-static int ACR83_GetFirmwareVersion(unsigned int reader_index, unsigned int *pFirmwareVersion);
+static int ACR83_GetFirmwareVersion(unsigned int reader_index, unsigned int *pVersion1, unsigned int *pVersion2);
 static int ACR83_DisplayLcdMessage(unsigned int reader_index, const char *message);
 static int ACR1222_GetFirmwareVersion(unsigned int reader_index, char *firmwareVersion, unsigned int *pFirmwareVersionLen);
 
@@ -83,6 +79,15 @@ int ccid_open_hack_pre(unsigned int reader_index)
 			ccid_descriptor->dwMaxDataRate = 9600;
 			break;
 
+		case ElatecTWN4:
+			/* use a timeout of 400 ms instead of 100 ms in CmdGetSlotStatus()
+			 * used by CreateChannelByNameOrChannel()
+			 * The reader answers after 280 ms if no tag is present */
+		case SCM_SCL011:
+			/* The SCM SCL011 reader needs 350 ms to answer */
+			ccid_descriptor->readTimeout = DEFAULT_COM_READ_TIMEOUT * 4;
+			break;
+
 		case ACS_ACR122U:
 		case ACS_AET62_PICC_READER:
 		case ACS_AET62_1SAM_PICC_READER:
@@ -482,10 +487,20 @@ int ccid_open_hack_post(unsigned int reader_index)
 		case HP_CCIDSMARTCARDKEYBOARD:
 		case FUJITSUSMARTKEYB:
 			/* the Secure Pin Entry is bogus so disable it
-			 * http://martinpaljak.net/2011/03/19/insecure-hp-usb-smart-card-keyboard/
+			 * https://web.archive.org/web/20120320001756/http://martinpaljak.net/2011/03/19/insecure-hp-usb-smart-card-keyboard/
+			 *
+			 * The problem is that the PIN code entered using the Secure
+			 * Pin Entry function is also sent to the host.
 			 */
 			ccid_descriptor->bPINSupport = 0;
 			break;
+		case HID_AVIATOR:
+			/* The chip advertises pinpad but actually doesn't have one */
+			ccid_descriptor->bPINSupport = 0;
+			/* Firmware uses chaining */
+			ccid_descriptor->dwFeatures &= ~CCID_CLASS_EXCHANGE_MASK;
+			ccid_descriptor->dwFeatures |= CCID_CLASS_EXTENDED_APDU;
+			break;
 
 #if 0
 		/* SCM SCR331-DI contactless */
@@ -525,6 +540,26 @@ int ccid_open_hack_post(unsigned int reader_index)
 			}
 			break;
 #endif
+		case CHERRY_KC1000SC:
+			if ((0x0100 == ccid_descriptor->IFD_bcdDevice)
+				&& (ccid_descriptor->dwFeatures & CCID_CLASS_EXCHANGE_MASK) == CCID_CLASS_SHORT_APDU)
+			{
+				/* firmware 1.00 is bogus
+				 * With a T=1 card and case 2 APDU (data from card to
+				 * host) the maximum size returned by the reader is 128
+				 * byes. The reader is then using chaining as with
+				 * extended APDU.
+				 */
+				ccid_descriptor->dwFeatures &= ~CCID_CLASS_EXCHANGE_MASK;
+				ccid_descriptor->dwFeatures |= CCID_CLASS_EXTENDED_APDU;
+			}
+			break;
+
+		case ElatecTWN4:
+		case SCM_SCL011:
+			/* restore default timeout (modified in ccid_open_hack_pre()) */
+			ccid_descriptor->readTimeout = DEFAULT_COM_READ_TIMEOUT;
+			break;
 
 		case ACS_ACR33U_A1_3SAM_ICC_READER:
 		case ACS_ACR33U_A2_3SAM_ICC_READER:
@@ -542,7 +577,7 @@ int ccid_open_hack_post(unsigned int reader_index)
 				ccid_descriptor->wLcdLayout = 0x0210;
 
 				DEBUG_INFO1("Getting ACR83U firmware version...");
-				if (ACR83_GetFirmwareVersion(reader_index, &firmwareVersion))
+				if (ACR83_GetFirmwareVersion(reader_index, &firmwareVersion, NULL))
 				{
 					DEBUG_INFO2("ACR83U firmware version: 0x%04X", firmwareVersion);
 					if (firmwareVersion >= 0x4500)
@@ -565,7 +600,6 @@ int ccid_open_hack_post(unsigned int reader_index)
 
 		case ACS_APG8201:
 		case ACS_APG8201Z:
-		case ACS_APG8201Z2:
 			ccid_descriptor->wLcdLayout = 0x0210;
 
 			// APG8201 uses short APDU exchange
@@ -576,6 +610,30 @@ int ccid_open_hack_post(unsigned int reader_index)
 			}
 			break;
 
+		case ACS_APG8201Z2:
+			{
+				unsigned int version1 = 0;
+				unsigned int version2 = 0;
+
+				ccid_descriptor->wLcdLayout = 0x0210;
+
+				DEBUG_INFO1("Getting APG8201Z2 firmware version...");
+				if (ACR83_GetFirmwareVersion(reader_index, &version1,
+					&version2))
+				{
+					DEBUG_INFO3("APG8201Z2 firmware version: 0x%04X%04X",
+						version1, version2);
+
+					/* Disable SetParameters for 029Z. */
+					if ((version1 == 0x3032) && (version2 == 0x395A))
+					{
+						ccid_descriptor->dwFeatures &= ~CCID_CLASS_AUTO_PPS_CUR;
+						ccid_descriptor->dwFeatures |= CCID_CLASS_AUTO_PPS_PROP;
+					}
+				}
+			}
+			break;
+
 		case ACS_ACR85_PINPAD_READER_ICC:
 			{
 				unsigned int firmwareVersion;
@@ -585,7 +643,7 @@ int ccid_open_hack_post(unsigned int reader_index)
 				ccid_descriptor->dwFeatures |= CCID_CLASS_SHORT_APDU;
 
 				DEBUG_INFO1("Getting ACR85 ICC firmware version...");
-				if (ACR83_GetFirmwareVersion(reader_index, &firmwareVersion))
+				if (ACR83_GetFirmwareVersion(reader_index, &firmwareVersion, NULL))
 				{
 					DEBUG_INFO2("ACR85 ICC firmware version: 0x%04X", firmwareVersion);
 
@@ -1054,7 +1112,7 @@ void EnablePicc(unsigned int reader_index, int enabled)
 	}
 }
 
-static int ACR83_GetFirmwareVersion(unsigned int reader_index, unsigned int *pFirmwareVersion)
+static int ACR83_GetFirmwareVersion(unsigned int reader_index, unsigned int *pVersion1, unsigned int *pVersion2)
 {
 	int ret = 0;
 	unsigned char command[] = { 0x04, 0x00, 0x00, 0x00, 0x00 };
@@ -1064,9 +1122,18 @@ static int ACR83_GetFirmwareVersion(unsigned int reader_index, unsigned int *pFi
 
 	if (CmdEscape(reader_index, command, commandLen, response, &responseLen, 0) == IFD_SUCCESS)
 	{
-		if ((responseLen >= 7) && (response[0] == 0x84))
+		if ((responseLen >= 9) && (response[0] == 0x84))
 		{
-			*pFirmwareVersion = (response[5] << 8) | response[6];
+			if (pVersion1 != NULL)
+			{
+				*pVersion1 = (response[5] << 8) | response[6];
+			}
+
+			if (pVersion2 != NULL)
+			{
+				*pVersion2 = (response[7] << 8) | response[8];
+			}
+
 			ret = 1;
 		}
 	}
