@@ -1,7 +1,7 @@
 /*
 	ccid_usb.c: USB access routines using the libusb library
 	Copyright (C) 2003-2010	Ludovic Rousseau
-	Copyright (C) 2009-2015	Advanced Card Systems Ltd.
+	Copyright (C) 2009-2016	Advanced Card Systems Ltd.
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Lesser General Public
@@ -18,10 +18,6 @@
 	Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-/*
- * $Id$
- */
-
 #define __CCID_USB__
 
 #include <stdio.h>
@@ -204,6 +200,9 @@ static void close_libusb_if_needed(void)
 {
 	int i, to_exit = TRUE;
 
+	if (NULL == ctx)
+		return;
+
 	/* if at least 1 reader is still in use we do not exit libusb */
 	for (i=0; i<CCID_DRIVER_MAX_READERS; i++)
 	{
@@ -244,6 +243,8 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 	char infofile[FILENAME_MAX];
 #ifndef __APPLE__
 	unsigned int device_vendor, device_product;
+	unsigned int device_bus = 0;
+	unsigned int device_addr = 0;
 #else
 	/* 100 ms delay */
 	struct timespec sleep_time = { 0, 100 * 1000 * 1000 };
@@ -289,9 +290,11 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 		 */
 		if ((dirname = strstr(device, "libudev:")) != NULL)
 		{
-			/* convert the interface number */
-			interface_number = atoi(dirname + 8 /* "libudev:" */);
-			DEBUG_COMM2("interface_number: %d", interface_number);
+			/* convert the interface number, bus and device ids */
+			if (sscanf(dirname + 8, "%d:/dev/bus/usb/%d/%d", &interface_number, &device_bus, &device_addr) == 3) {
+				DEBUG_COMM2("interface_number: %d", interface_number);
+				DEBUG_COMM3("usb bus/device: %d/%d", device_bus, device_addr);
+			}
 		}
 	}
 #endif
@@ -340,24 +343,13 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 		}
 	}
 
-#ifdef __APPLE__
-again_libusb:
-#endif
-	cnt = libusb_get_device_list(ctx, &devs);
-	if (cnt < 0)
-	{
-		DEBUG_CRITICAL("libusb_get_device_list() failed\n");
-		return_value = STATUS_UNSUCCESSFUL;
-		goto end1;
-	}
-
 #define GET_KEYS(key, values) \
 	rv = LTPBundleFindValueWithKey(&plist, key, values); \
 	if (rv) \
 	{ \
 		DEBUG_CRITICAL2("Value/Key not defined for " key " in %s", infofile); \
 		return_value = STATUS_UNSUCCESSFUL; \
-		goto end2; \
+		goto end1; \
 	}
 
 	GET_KEYS("ifdVendorID", &ifdVendorID)
@@ -373,6 +365,17 @@ again_libusb:
 		goto end1;
 	}
 
+#ifdef __APPLE__
+again_libusb:
+#endif
+	cnt = libusb_get_device_list(ctx, &devs);
+	if (cnt < 0)
+	{
+		DEBUG_CRITICAL("libusb_get_device_list() failed\n");
+		return_value = STATUS_UNSUCCESSFUL;
+		goto end1;
+	}
+
 	/* for any supported reader */
 	for (alias=0; alias<list_size(ifdVendorID); alias++)
 	{
@@ -403,6 +406,16 @@ again_libusb:
 			uint8_t bus_number = libusb_get_bus_number(dev);
 			uint8_t device_address = libusb_get_device_address(dev);
 
+#ifndef __APPLE__
+			if ((device_bus || device_addr)
+				&& ((bus_number != device_bus)
+				|| (device_address != device_addr))) {
+				/* not USB the device we are looking for */
+				continue;
+			}
+#endif
+			DEBUG_COMM3("Try device: %d/%d", bus_number, device_address);
+
 			int r = libusb_get_device_descriptor(dev, &desc);
 			if (r < 0)
 			{
@@ -411,6 +424,8 @@ again_libusb:
 				continue;
 			}
 
+			DEBUG_COMM3("vid/pid : %04X/%04X", desc.idVendor, desc.idProduct);
+
 			if (desc.idVendor == vendorID && desc.idProduct == productID)
 			{
 				int already_used;
@@ -701,12 +716,24 @@ again:
 				}
 
 #ifdef USE_COMPOSITE_AS_MULTISLOT
-				/* use the next interface for the next "slot" */
-				static_interface++;
+				if ((GEMALTOPROXDU == readerID)
+					|| (GEMALTOPROXSU == readerID)
+					|| (FEITIANR502DUAL == readerID)
+					|| (ACS_ACR1281_DUAL_READER_QPBOC == readerID)
+					|| (ACS_ACR1281_DUAL_READER_BSI == readerID)
+					|| (ACS_ACR1281_1S_PICC_READER == readerID)
+					|| (ACS_ACR1251_1S_CL_READER == readerID)
+					|| (ACS_ACR1251U_C == readerID)
+					|| (ACS_ACR1251K_DUAL_READER == readerID)
+					|| (ACS_ACR1252_1S_CL_READER == readerID))
+				{
+					/* use the next interface for the next "slot" */
+					static_interface++;
 
-				/* reset for a next reader */
-				if (static_interface > 2)
-					static_interface = (FEITIANR502DUAL == readerID) ? 0: 1;
+					/* reset for a next reader */
+					if (static_interface > 2)
+						static_interface = (FEITIANR502DUAL == readerID) ? 0: 1;
+				}
 #endif
 
 				/* Get Endpoints values*/
@@ -1017,7 +1044,9 @@ end:
 			goto again_libusb;
 		}
 #endif
+		/* failed */
 		close_libusb_if_needed();
+
 		if (claim_failed)
 			return STATUS_COMM_ERROR;
 		DEBUG_INFO1("Device not found?");
@@ -1036,6 +1065,9 @@ end1:
 	/* free bundle list */
 	bundleRelease(&plist);
 
+	if (return_value != STATUS_SUCCESS)
+		close_libusb_if_needed();
+
 	return return_value;
 } /* OpenUSBByName */
 
@@ -1332,10 +1364,6 @@ status_t CloseUSB(unsigned int reader_index)
 		if (usbDevice[reader_index].ccid.sIFD_iManufacturer)
 			free(usbDevice[reader_index].ccid.sIFD_iManufacturer);
 
-		/* reset so that bSeq starts at 0 again */
-		if (DriverOptions & DRIVER_OPTION_RESET_ON_CLOSE)
-			(void)libusb_reset_device(usbDevice[reader_index].dev_handle);
-
 		if (usbDevice[reader_index].ccid.arrayOfSupportedDataRates)
 			free(usbDevice[reader_index].ccid.arrayOfSupportedDataRates);
 
@@ -1475,7 +1503,8 @@ static int get_end_points(struct libusb_config_descriptor *desc,
 		/* CCID Class? */
 		if (desc->interface[i].altsetting->bInterfaceClass == 0xb
 #ifdef ALLOW_PROPRIETARY_CLASS
-			|| desc->interface[i].altsetting->bInterfaceClass == 0xff
+			|| (desc->interface[i].altsetting->bInterfaceClass == 0xff
+			&& 54 == desc->interface[i].altsetting->extra_length)
 
 			// bInterfaceClass is 0x00 in ACR83U, ACR88U and ACR128U
 			|| desc->interface[i].altsetting->bInterfaceClass == 0x00
@@ -1679,7 +1708,8 @@ int InterruptRead(int reader_index, int timeout /* in ms */)
 	ret = libusb_submit_transfer(transfer);
 	if (ret < 0) {
 		libusb_free_transfer(transfer);
-		DEBUG_CRITICAL2("libusb_submit_transfer failed: %d", ret);
+		DEBUG_CRITICAL2("libusb_submit_transfer failed: %s",
+			libusb_error_name(ret));
 		return ret;
 	}
 
@@ -1687,17 +1717,18 @@ int InterruptRead(int reader_index, int timeout /* in ms */)
 
 	while (!completed)
 	{
-		ret = libusb_handle_events(ctx);
+		ret = libusb_handle_events_completed(ctx, &completed);
 		if (ret < 0)
 		{
 			if (ret == LIBUSB_ERROR_INTERRUPTED)
 				continue;
 			libusb_cancel_transfer(transfer);
 			while (!completed)
-				if (libusb_handle_events(ctx) < 0)
+				if (libusb_handle_events_completed(ctx, &completed) < 0)
 					break;
 			libusb_free_transfer(transfer);
-			DEBUG_CRITICAL2("libusb_handle_events failed: %d", ret);
+			DEBUG_CRITICAL2("libusb_handle_events failed: %s",
+				libusb_error_name(ret));
 			return ret;
 		}
 	}
@@ -1841,7 +1872,7 @@ static void *Multi_PollingProc(void *p_ext)
 		completed = 0;
 		while (!completed && !msExt->terminated)
 		{
-			rv = libusb_handle_events(ctx);
+			rv = libusb_handle_events_completed(ctx, &completed);
 			if (rv < 0)
 			{
 				DEBUG_COMM2("libusb_handle_events err %d", rv);
@@ -1853,7 +1884,7 @@ static void *Multi_PollingProc(void *p_ext)
 
 				while (!completed && !msExt->terminated)
 				{
-					if (libusb_handle_events(ctx) < 0)
+					if (libusb_handle_events_completed(ctx, &completed) < 0)
 						break;
 				}
 
@@ -1874,6 +1905,15 @@ static void *Multi_PollingProc(void *p_ext)
 
 			libusb_free_transfer(transfer);
 
+			/* Break if the device was disconnected. */
+			if (status == LIBUSB_TRANSFER_NO_DEVICE)
+			{
+				DEBUG_COMM3("Multi_PollingProc (%d/%d): Device disconnected",
+					usbDevice[msExt->reader_index].bus_number,
+					usbDevice[msExt->reader_index].device_address);
+				break;
+			}
+
 			switch (status)
 			{
 				case LIBUSB_TRANSFER_COMPLETED:
@@ -2220,7 +2260,7 @@ static void *CardDetectionThread(void *pParam)
 		completed = 0;
 		while (!completed && !usbDevice[reader_index].terminated)
 		{
-			rv = libusb_handle_events(ctx);
+			rv = libusb_handle_events_completed(ctx, &completed);
 			if (rv < 0)
 			{
 				DEBUG_COMM2("libusb_handle_events err %d", rv);
@@ -2232,7 +2272,7 @@ static void *CardDetectionThread(void *pParam)
 
 				while (!completed && !usbDevice[reader_index].terminated)
 				{
-					if (libusb_handle_events(ctx) < 0)
+					if (libusb_handle_events_completed(ctx, &completed) < 0)
 						break;
 				}
 
